#!/bin/sh
# $Id: t-mt-4.sh,v 1.1 2006/11/09 04:47:50 ca Exp $
#
# ------------------------------------------------------------
# Description:
# mailertable routing tests by contacting SMAR
# uses t-smar-1 and startsmar.sh
# ------------------------------------------------------------
#
test -s stop && exit 1
#

# path to programs
P=..
SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../smar
fi
. ${SD}/common.sh

OUT=mt-4.out
IP=mt-4.ip
OK=mt-4.ok
CONF=mt.conf
TPRG=./t-smar-1
VERBOSE=false

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

rm -f ${SOCK} ${PIDS}

# prepare mailertable
${CAT} > ${MT} <<EOF
host123.meta1.org	[1.2.3.0]
meta1.org	[1.2.3.2]
.meta1.org	[1.2.3.4]
.org		[1.2.3.6]
.		[1.2.3.9]
rcpt+detail@meta1.org	[1.2.3.10]
rcpt++@meta1.org	[1.2.3.11]
rcpt+*@meta1.org	[1.2.3.12]
rcpt*@meta1.org	[1.2.3.13]
rcpt@meta1.org	[1.2.3.14]
EOF

rm -f ${MTMAPDB}
${MM} -t'	' -F ${MTMAPDB} < ${MT}

${CAT} > ${CONF} <<EOF
smar {
 map mt { type=hash; file=${MTMAPDB}; }
 mailertable { name=mt;
	flags= {full_address, detail_plus, detail_star, star} }
}
EOF

# start SMAR
${SHELL} ${SD}/startsmar.sh -o '-d 4' -f ${CONF} || exit 1

rm -f ${OUT} ${IP} ${OK}


TEST=1
# ----------------------------------------
${VALGRIND} ${TPRG} -M -VVVV '<rcpt@meta1.org>' > ${OUT} 2>&1
grep '^IP' ${OUT} | sort > ${IP}
cat > ${OK} <<EOF
IP[0]=1.2.3.14 [102030e]
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi

TEST=2
# ----------------------------------------
${VALGRIND} ${TPRG} -M -VVVV '<rcpt+detail@meta1.org>' > ${OUT} 2>&1
grep '^IP' ${OUT} | sort > ${IP}
cat > ${OK} <<EOF
IP[0]=1.2.3.10 [102030a]
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi

TEST=3
# ----------------------------------------
${VALGRIND} ${TPRG} -M -VVVV '<rcpt+other@meta1.org>' > ${OUT} 2>&1
grep '^IP' ${OUT} | sort > ${IP}
cat > ${OK} <<EOF
IP[0]=1.2.3.11 [102030b]
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi

TEST=4
# ----------------------------------------
${VALGRIND} ${TPRG} -M -VVVV '<rcpt+@meta1.org>' > ${OUT} 2>&1
grep '^IP' ${OUT} | sort > ${IP}
cat > ${OK} <<EOF
IP[0]=1.2.3.12 [102030c]
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi

# rm -f ${OUT} ${IP} ${OK}

# stop SMAR
for i in `cat ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi

exit ${ERRS}
